; PROGRAM NAME	:	Temperature Data Logger
; FILE NAME	:	Temp_Log.asm

; SOFTWARE BY	:	SIMON BRAMBLE
;			simon_bramble@maximhq.com
; DATE		: 	3rd March 2001
; DEVELOPMENT TIME	10 Grolsch
; PROCESSOR	:	AT90S2313
; CLOCK SPEED	:	32.768kHz external XTAL

; DESCRIPTION
;
; This program is part of a temperature data logger that measures the 
; ambient temperature every 10 minutes

; It reads the input on the R/W line of Port B If low, it sleeps, then wakes up
; on Timer overflow. If high, goes into the playback routine and downloads 
; the data via a MAX3100 and MAX202 to a PC
;
; The temperature transducer is the MAX6576, which gives an output period of 
; (160us/Degree Kelvin. With the prescaler of Timer 0 set to 8, this allows a 
; MAX temp of 118 deg C
;
; The result is stored in an Atmel E2 memory, AT24C12810PC-2.7.

.device AT90S2313

;***************************************************************************
; PORT DEFINITIONS - this is just a dump of the '2313 include file
;***************************************************************************

; PORTB assignments
;	b7	b6	b5	b4	b3	b2	b1	b0
;			6576Vcc	R/W	6576i/p	sda	scl	

; PORTD assignments - for UART download
;	b6	b5	b4	b3	b2	b1	b0
;				dout	din	sclk	cs
;***** I/O Register Definitions
.equ	SREG	=$3f
.equ	SPL	=$3d
.equ	TIMSK	=$39
.equ	MCUCR	=$35
.equ	TCCR0	=$33
.equ	TCNT0	=$32
.equ	TCCR1A	=$2f
.equ	TCCR1B	=$2e
.equ	TCNT1H	=$2d
.equ	TCNT1L	=$2c
.equ	PORTB	=$18				
.equ	DDRB	=$17
.equ	PINB	=$16
.equ	PORTD	=$12
.equ	DDRD	=$11
.equ	PIND	=$10

; PORTB pin assignments:
.equ	scl	=1				; I2C serial clock
.equ	sda	=2				; I2C serial data I/O
.equ	temp	=3				; MAX6576 temperature input
.equ	RW	=4				; Read from or /(Write to) Memory

; PORTD pin assignments:			; for UART download
.equ	cs	=0				; chip select
.equ	sclk	=1				; serial clock
.equ	din	=2				; data into processor
.equ	dout	=3				; data out of processor

;***** Bit Definitions

.equ    RAMEND  =$DF   				;Last On-Chip SRAM Location
.equ	OVF0addr=$006				;Overflow0 Interrupt Vector Address
.equ	timer	=0

;***************************************************************************
; REGISTER DEFINITIONS
;***************************************************************************

.def 	junk		=r16			; junk register
.def	degrees		=r17			; counter for MAX6576 period
.def	flag_reg	=r18			; general purpose flag reg
.def	count		=r19			; junk counter
.def	low_tx		=r20			; low byte of UART TX
.def	high_tx		=r21			; high byte of UART TX
.def	low_rx		=r22			; low byte of UART RX
.def	high_rx		=r23			; high byte of UART RX
.def	cont_byte	=r24			; I2C control byte
.def	lowadd		=r25			; I2C low address byte
.def	highadd		=r26			; I2C high address byte
.def	I2C_byte	=r27			; I2C byte manipulation
	
;***************************************************************************
; MAIN CODE
;***************************************************************************
.cseg
.org	0
	rjmp	reset				; executed on RESET
.org	5					
	rjmp	Int_T1				; executed on Timer 1 overflow
start:

	sbic	PINB, RW			; skip if in Write (Record) mode
	rjmp	playback			; Play back data from memory

;'Record' Section
	ldi	junk, 0b00100000		; activate Sleep/Idle mode
	out	MCUCR, junk			; 
	sleep					; Idle Mode
	nop					; wake up on Timer 1 overflow
	rjmp	start				; return to start

;'Playback' section
playback:					; Send results to PC 
	rcall	delay				; allow MAX3100 oscillator to stabilise
	rcall	uart_init			; initialise UART
playloop:
	rcall	readmem				; read I2C memory
	mov	low_tx, low_rx			; ready for uart routine
	rcall	wr_data				; load UART. Squirt to PC

	rcall	delay				; wait 10 ms before next TX
	rcall	inc_mem				; increment address

	cpi	highadd, 0x40			; is memory about to overflow?
	brne	playloop

	rjmp	end

;***************************************************************************
; SUBROUTINES
;***************************************************************************

;********************** MICRO INITIALISATION ROUTINE ***********************
reset:						; port definitions and reset
; initialise Stack:
	ldi	junk, low(RAMEND)		; 
	out	SPL, junk			; define top of Stack

; initialise Timer 0
	ldi	junk, 0b00000010		; Prescaler = 8
	out	TCCR0, junk

; initialise Timer 1
	clr	junk
	out	TCCR1A, junk			; disable PWM mode
	ldi	junk, 0b00000101		; Prescaler = CLK/1024
	out	TCCR1B, junk
	ldi	junk, $B5			; this causes overflow after 10 mins
	out	TCNT1H, junk			; clear counter registers
	clr	junk
	out	TCNT1L, junk

; initialise ports:
	ldi	junk, 0b11100111		; PB3, PB4 = inputs
	out	DDRB,junk			; 
	clr	junk
	out	PORTB, junk			; clear outputs and float inputs

	ldi	junk, 0b11111011		; PD2 = input
	out	DDRD,junk			; 
	clr	junk
	out	PORTD, junk			; clear outputs and float inputs

	sbi	PORTB, sda			; initialise sda with scl high
	sbi	PORTB, scl
	sbi	PORTD, CS			; set chip select

	ldi	cont_byte, 0b10100000		; control byte for memory

; initialise registers
	clr	flag_reg
	clr	lowadd	
	clr	highadd

	sbic	PINB, RW			; do not set interrupts 
	rjmp	start				; if in readback mode
						
; initialise Interrupts:
	ldi	junk, 0b10000000		; enable Timer 1
	out	TIMSK, junk			; overflow Interrupt
	out	SREG, junk			; set Global Interrupt bit

	rjmp	start				; return to main program 

;*************** TIMER/COUNTER OVERFLOW INTERRUPT ROUTINE ****************
Int_T1:						; arrive here on Interrupt

	sbi	PORTB, 5			; power up MAX6576

	clr	junk				; clear Sleep mode
	out	MCUCR, junk			; 

	ldi	junk, $B5			; reset Timer1
	out	TCNT1H, junk
	clr	junk
	out	TCNT1L, junk

; look for the first rising edge of the MAX6576 output	

hitest:						; looks for high to low edge
	sbic	PINB, temp			; skip if 6576 I/O is low
	rjmp	hitest				; look again

lowtest:					; looks for low to high edge
	sbis	PINB, temp			; skip if 6576 I/O is high
	rjmp	lowtest
						; we are now at rising edge of 6576 O/P 
	sbrs	flag_reg, timer			; skip if timer is ON	

	rjmp	timeron				; start timer
	rjmp	timeroff			; stop timer and output result

; start timer:
timeron:
	clr	junk
	out	TCNT0, junk			; resets Timer 0
	sbr	flag_reg,1 			; set 'timer on' bit (bit 0)
	rjmp	hitest				; looks for next rising edge from 6576

; stop timer and output result to memory:
timeroff:
	cbr	flag_reg, 1			; clears 'timer on' bit
	in	junk, TCNT0			; load timer into junk
	mov	degrees, junk			; period output of 6576

	rcall	writemem			; loads data into memory

	cbi	PORTB, 5			; power down MAX6576
	reti					; return to sleep and wait another 10 mins	

;****************** LOAD MAX6576 DELAY INTO I2C MEMORY *******************

writemem:					; loads data into memory

	cbi	PORTB, sda			; START condition
	cbr	cont_byte, 1			; set write mode

	mov	I2C_byte, cont_byte 
	rcall	I2C_write			; load control byte	
	rcall	ackn_rx				; wait for acknowledge

	mov	I2C_byte, highadd
	rcall	I2C_write			; load address byte
	rcall	ackn_rx				; wait for acknowledge

	mov	I2C_byte, lowadd
	rcall	I2C_write			; load address byte
	rcall	ackn_rx				; wait for acknowledge

	mov	I2C_byte, degrees
	rcall	I2C_write			; load data
	rcall	ackn_rx				; wait for acknowledge

	sbi	PORTB, scl
	sbi	PORTB, sda			; STOP condition

	rcall	delay				; 10ms E2 write time
	rcall	inc_mem				; increment memory address

	cpi	highadd, 0x40			; is memory about to overflow?
	breq	memend				; cannot jump >63 bytes
	ret
memend:
	rjmp	end				; set micro to sleep

;*************************** READ I2C MEMORY  ****************************

readmem:
	cbi	PORTB, sda			; START condition
	cbr	cont_byte, 1			; set write mode

	mov	I2C_byte, cont_byte 
	rcall	I2C_write			; load control byte	
	rcall	ackn_rx				; wait for acknowledge

	mov	I2C_byte, highadd
	rcall	I2C_write			; load address byte
	rcall	ackn_rx				; wait for acknowledge

	mov	I2C_byte, lowadd
	rcall	I2C_write			; load address byte
	rcall	ackn_rx				; wait for acknowledge

	sbi	PORTB, sda
	sbi	PORTB, scl
	cbi	PORTB, sda			; START condition for Random Read

	sbr	cont_byte, 1			; set read mode (bit 0)
	mov	I2C_byte, cont_byte 
	rcall	I2C_write			; load control byte	
	rcall	ackn_rx				; wait for acknowledge

	rcall	I2C_read			; read in byte from memory
	rcall	ackn_tx				; issue acknowledge

	sbi	PORTB, scl
	sbi	PORTB, sda			; STOP condition

	ret

;************************** I2C WRITE ROUTINE  ***************************

I2C_write:
	ldi	count,0x8  
writeloop:
	cbi	PORTB, scl
	cbi	PORTB, sda
	sbrc	I2C_byte, 7			; check MSB of I2C_byte
	sbi	PORTB, sda
	sbi	PORTB, scl
	rol	I2C_byte
	dec	count
	brne	writeloop			; return to writeloop
	cbi	PORTB, scl
	cbi	PORTB, sda

; On the last falling edge of PORTB, SCL, memory takes over the SDA line
; and drives it low (ackn). The above comment ensures the micro is also
; transmitting a logic 0 on the changeover.

; you leave I2C_write with SCL low

	ret

;*************************** I2C READ ROUTINE  ***************************

I2C_read:

	cbi	DDRB, sda			; set sda to input
	ldi	count, 0x8
readloop:	
	clc					; precondition Carry bit
	rol	low_rx
	sbi	PORTB, scl			; clocks out MSB
	sbic	PINB, sda
	sbr	low_rx, 1			; sets bit 0  
	cbi	PORTB, scl
	dec	count
	brne	readloop

; on the last falling edge of SCL (above), the SDA line is left floating
; so the line is immediately pulled high by the SDA resistor.

; you leave I2C_read with SCL low

	ret

;********************** RECEIVE ACKNOWLEDGE ROUTINE ***********************

ackn_rx:	
	cbi	DDRB, sda			; set sda to input
	sbi	PORTB, scl
	sbic	PINB, sda
	rjmp	end				; no acknowledge cometh
	cbi	PORTB, scl			; clears ackn

; here the bus is floating and pulled high by the sda resistor

	sbi	DDRB, sda			; set sda to output
	ret

; you leave ackn_rx with SCL low

;********************** TRANSMIT ACKNOWLEDGE ROUTINE ***********************

ackn_tx:
	sbi	DDRB, sda			; set sda to output

	sbi	PORTB, sda
	sbi	PORTB, scl			; issue acknowledge for next address
	cbi	PORTB, scl
	cbi	PORTB, sda
	ret

; you leave ackn_tx with SCL low

;****************************** DELAY ROUTINE ******************************

delay:						; 10ms approx
	ldi	junk, 163
dlylp:	dec	junk
	brne	dlylp				; loop to dlylp if not zero
	ret
;*********************** INCREMENT ADDRESS ROUTINE *************************

inc_mem:
	inc	lowadd
	brne	still_lo			; increment highadd if lowadd=0
	inc	highadd
still_lo:
	ret

;*************************** UART TX/RX ROUTINE  ***************************

; this is s genral UART TX/RX routine, although we are only writing data here.
; You can butcher this code for other applications involving TX & RX on the Maxim UARTs 

uart_init:					; general UART I/O routine
	ldi	low_tx, 0b00001011		; 9600, 8 bit, no parity 1 stop bit
wr_config:
	ldi	high_tx, 0b11100000		; No Interrupts from UART
	rjmp	uart_io
rd_config:
	ldi	high_tx, 0b01000000
	rjmp	uart_io
wr_data:
	ldi	high_tx, 0b10000000
	rjmp	uart_io
rd_data:
	ldi	high_tx, 0b00000000
uart_io:
	cbi	PORTD, CS			; start SPI frame
	ldi	count, 0x10
uartloop:
	cbi	PORTD, DOUT			; reset Port D, Dout
	sbrc	high_tx,7			; skip next instruction if MSB is set
	sbi	PORTD, DOUT
	sbi	PORTD, SCLK
	clc
	sbic	PIND, DIN
	sec					; set carry bit

	rol	low_rx				; rotates DIN into low_rx LSB
	rol	high_rx				; rotates MSB low_ into LSB high_
	rol	low_tx				; rotate random data into LSB
	rol	high_tx				; rotate MSB low_ into LSB high_

	cbi	PORTD, SCLK			; clock out next bit of data from UART

	dec	count
	brne	uartloop
	sbi	PORTD, CS			; frames data
	ret

end:
;	clr	junk
	out	SREG, junk			; clear interrupt flag
; temp code *****************************
	sbi	PORTB, 7
	cbi	PORTB, 7
; temp code *****************************
;	sleep					; Rip van Winkle
	rjmp	end

;***************************************************************************
; END OF FILE
;***************************************************************************